import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:syncrow_web/pages/analytics/models/air_quality_data_model.dart';
import 'package:syncrow_web/pages/analytics/modules/energy_management/helpers/energy_management_charts_helper.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class AqiDistributionChart extends StatelessWidget {
  const AqiDistributionChart({super.key, required this.chartData});
  final List<AirQualityDataModel> chartData;

  static const _rodStackItemsSpacing = 0.4;
  static const _barWidth = 13.0;
  static final _barBorderRadius = BorderRadius.circular(22);

  @override
  Widget build(BuildContext context) {
    final sortedData = List<AirQualityDataModel>.from(chartData)
      ..sort(
        (a, b) => a.date.compareTo(b.date),
      );

    return BarChart(
      BarChartData(
        maxY: 100.1,
        gridData: EnergyManagementChartsHelper.gridData(
          horizontalInterval: 20,
        ),
        borderData: EnergyManagementChartsHelper.borderData(),
        barTouchData: _barTouchData(context),
        titlesData: _titlesData(context),
        barGroups: _buildBarGroups(sortedData),
      ),
      duration: Duration.zero,
    );
  }

  List<BarChartGroupData> _buildBarGroups(List<AirQualityDataModel> sortedData) {
    return List.generate(sortedData.length, (index) {
      final data = sortedData[index];
      final stackItems = <BarChartRodData>[];
      double currentY = 0;
      bool isFirstElement = true;

      // Sort data by type to ensure consistent order
      final sortedPercentageData = List<AirQualityPercentageData>.from(data.data)
        ..sort((a, b) => a.type.compareTo(b.type));

      for (final percentageData in sortedPercentageData) {
        stackItems.add(
          BarChartRodData(
            fromY: currentY,
            toY: currentY + percentageData.percentage ,
            color: AirQualityDataModel.metricColors[percentageData.name]!,
            borderRadius: isFirstElement
                ? const BorderRadius.only(
                    topLeft: Radius.circular(22),
                    topRight: Radius.circular(22),
                  )
                : _barBorderRadius,
            width: _barWidth,
          ),
        );
        currentY += percentageData.percentage + _rodStackItemsSpacing;
        isFirstElement = false;
      }

      return BarChartGroupData(
        x: index,
        barRods: stackItems,
        groupVertically: true,
      );
    });
  }

  BarTouchData _barTouchData(BuildContext context) {
    return BarTouchData(
      touchTooltipData: BarTouchTooltipData(
        getTooltipColor: (_) => ColorsManager.whiteColors,
        tooltipBorder: const BorderSide(
          color: ColorsManager.semiTransparentBlack,
        ),
        tooltipRoundedRadius: 16,
        tooltipPadding: const EdgeInsets.all(8),
        getTooltipItem: (group, groupIndex, rod, rodIndex) {
          final data = chartData[group.x.toInt()];

          final List<TextSpan> children = [];

          final textStyle = context.textTheme.bodySmall?.copyWith(
            color: ColorsManager.blackColor,
            fontSize: 12,
          );

          // Sort data by type to ensure consistent order
          final sortedPercentageData = List<AirQualityPercentageData>.from(data.data)
            ..sort((a, b) => a.type.compareTo(b.type));

          for (final percentageData in sortedPercentageData) {
            children.add(TextSpan(
              text:
                  '\n${percentageData.type.toUpperCase()}: ${percentageData.percentage.toStringAsFixed(1)}%',
              style: textStyle,
            ));
          }

          return BarTooltipItem(
            DateFormat('dd/MM/yyyy').format(data.date),
            context.textTheme.bodyMedium!.copyWith(
              color: ColorsManager.blackColor,
              fontSize: 16,
              fontWeight: FontWeight.w600,
            ),
            children: children,
          );
        },
      ),
    );
  }

  FlTitlesData _titlesData(BuildContext context) {
    final titlesData = EnergyManagementChartsHelper.titlesData(
      context,
      leftTitlesInterval: 20,
    );

    final leftTitles = titlesData.leftTitles.copyWith(
      sideTitles: titlesData.leftTitles.sideTitles.copyWith(
        reservedSize: 70,
        interval: 20,
        maxIncluded: false,
        minIncluded: true,
        getTitlesWidget: (value, meta) => Padding(
          padding: const EdgeInsetsDirectional.only(end: 12),
          child: FittedBox(
            alignment: AlignmentDirectional.centerStart,
            fit: BoxFit.scaleDown,
            child: Text(
              '${value.toStringAsFixed(0)}%',
              style: context.textTheme.bodySmall?.copyWith(
                fontSize: 12,
                color: ColorsManager.lightGreyColor,
              ),
            ),
          ),
        ),
      ),
    );

    final bottomTitles = AxisTitles(
      sideTitles: SideTitles(
        showTitles: true,
        getTitlesWidget: (value, _) => FittedBox(
          alignment: AlignmentDirectional.bottomCenter,
          fit: BoxFit.scaleDown,
          child: Text(
            chartData[value.toInt()].date.day.toString(),
            style: context.textTheme.bodySmall?.copyWith(
              color: ColorsManager.lightGreyColor,
              fontSize: 8,
            ),
          ),
        ),
        reservedSize: 36,
      ),
    );

    return titlesData.copyWith(
      leftTitles: leftTitles,
      bottomTitles: bottomTitles,
    );
  }
}
